Imports System
Imports System.IO
Imports System.Drawing
Imports System.Drawing.Printing
Imports System.ComponentModel

Public Class PrintForm
    Inherits System.Windows.Forms.Form

#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Form overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    Friend WithEvents PrintButton As System.Windows.Forms.Button
    Friend WithEvents PreviewButton As System.Windows.Forms.Button
    Friend WithEvents StatusLabel As System.Windows.Forms.Label
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.PrintButton = New System.Windows.Forms.Button()
        Me.PreviewButton = New System.Windows.Forms.Button()
        Me.StatusLabel = New System.Windows.Forms.Label()
        Me.SuspendLayout()
        '
        'PrintButton
        '
        Me.PrintButton.Location = New System.Drawing.Point(224, 104)
        Me.PrintButton.Name = "PrintButton"
        Me.PrintButton.Size = New System.Drawing.Size(88, 24)
        Me.PrintButton.TabIndex = 0
        Me.PrintButton.Text = "Print"
        '
        'PreviewButton
        '
        Me.PreviewButton.Location = New System.Drawing.Point(56, 104)
        Me.PreviewButton.Name = "PreviewButton"
        Me.PreviewButton.Size = New System.Drawing.Size(88, 24)
        Me.PreviewButton.TabIndex = 1
        Me.PreviewButton.Text = "Preview"
        '
        'StatusLabel
        '
        Me.StatusLabel.Location = New System.Drawing.Point(40, 32)
        Me.StatusLabel.Name = "StatusLabel"
        Me.StatusLabel.Size = New System.Drawing.Size(264, 24)
        Me.StatusLabel.TabIndex = 2
        '
        'PrintForm
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(336, 172)
        Me.Controls.AddRange(New System.Windows.Forms.Control() {Me.StatusLabel, Me.PreviewButton, Me.PrintButton})
        Me.Name = "PrintForm"
        Me.Text = "Form1"
        Me.ResumeLayout(False)

    End Sub

#End Region
    ' private variables
    Private WithEvents _PDoc As PrintDocument
    Private _Data As StreamReader
    Private _currentPage As Integer = 0
    ' these will be created in BeginPrint and 
    ' destroyed in EndPrint.
    Private _headerFont As Font
    Private _bodyFont As Font
    Private _defaultBrush As Brush = Brushes.Black
    Private _defaultPen As New Pen(Brushes.Black, 0.25F)

    Private Sub PrintForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        _PDoc = New PrintDocument()
        ' this is what will show up in the print manager
        _PDoc.DocumentName = "printTest -- Northwind Contact List"
        ' make sure at least one printer is installed
        If _PDoc.PrinterSettings.PrinterName = "<no default printer>" Then
            StatusLabel.Text = "At least one (1) printer must be installed to continue."
            PrintButton.Enabled = False
            PreviewButton.Enabled = False
        Else
            StatusLabel.Text = "You have " + CInt(PrinterSettings.InstalledPrinters.Count).ToString + " printer(s) installed."
        End If
    End Sub

    Private Sub PreviewButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles PreviewButton.Click
        Try
            Dim previewDialog As New PrintPreviewDialog()

            ' display a pagesetup dialog
            Dim pageSetup As New PageSetupDialog()

            pageSetup.Document = _PDoc
            Dim Rc As DialogResult = pageSetup.ShowDialog()
            If Rc = DialogResult.Cancel Then Return

            ' display the preview dialog

            previewDialog.Document = _PDoc
            previewDialog.PrintPreviewControl.Zoom = 1.0
            previewDialog.WindowState = FormWindowState.Maximized
            previewDialog.ShowInTaskbar = True
            previewDialog.ShowDialog()
        Catch ex As Exception
            MsgBox(ex.Message & vbCrLf & ex.StackTrace)
        End Try
    End Sub

    Private Sub _PDoc_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles _PDoc.PrintPage
        _currentPage += 1
        Dim headerText As String = "Northwinds Customer Contacts"
        Dim mi As MarginInfo
        Dim leftMargin As Single = 0
        Dim rightMargin As Single = 0
        Dim topMargin As Single = 0
        Dim bottomMargin As Single = 0
        Dim pageHeight As Single = 0
        Dim pageWidth As Single = 0
        ' retrieve the hard printer margins
        Dim hDC As IntPtr = e.Graphics.GetHdc()

        Try
            mi = New marginInfo(hDc.ToInt32())
        Finally
            e.Graphics.ReleaseHdc(hDC)
        End Try

        e.Graphics.PageUnit = GraphicsUnit.Inch
        e.Graphics.PageScale = 0.01
        e.Graphics.TranslateTransform(-mi.Left, -mi.Top)

        ' retrieve the margins
        leftMargin = e.MarginBounds.Left - mi.Left
        rightMargin = e.MarginBounds.Right
        topMargin = e.MarginBounds.Top - mi.Top
        bottomMargin = e.MarginBounds.Bottom
        pageHeight = bottomMargin - topMargin
        pageWidth = rightMargin - leftMargin

        ' used to define the sections of the document
        Dim headerHeight As Single = _headerFont.GetHeight(e.Graphics)
        Dim footerHeight As Single = _bodyFont.GetHeight(e.Graphics)

        ' report header
        Dim ReportheaderR As New RectangleF(leftMargin, topMargin, pageWidth, headerHeight)
        ' report body
        Dim bodyR As New RectangleF(leftMargin, ReportheaderR.Bottom, pageWidth, pageHeight - ReportheaderR.Height - footerHeight)
        ' report footer
        Dim ReportFooterR As New RectangleF(leftMargin, bodyR.Bottom, pageWidth, footerHeight * 2)

        ' uncomment these lines to draw borders around the rectangles

        'drawRect(e.Graphics, _defaultPen, leftMargin, topMargin, pageWidth, headerHeight)
        'drawRect(e.Graphics, _defaultPen, leftMargin, ReportheaderR.Bottom, pageWidth, pageHeight - ReportheaderR.Height - footerHeight)
        'drawRect(e.Graphics, _defaultPen, leftMargin, bodyR.Bottom, pageWidth, ReportfooterR.Height)

        'centerText(e.Graphics, "Header section", _headerFont, _defaultBrush, ReportheaderR)
        'centerText(e.Graphics, "Body section", _headerFont, _defaultBrush, bodyR)
        'centerText(e.Graphics, "Footer section", _headerFont, _defaultBrush, ReportfooterR)
        'return

        ' results of using the Split function on the text
        Dim data() As String
        ' a line of text from our file
        Dim text As String = ""


        ' print the header once per page
        centerText(e.Graphics, headerText, _headerFont, _defaultBrush, ReportheaderR)

        ' how many lines can we fit on a page?              
        Dim bodyFontHeight As Single = footerHeight
        Dim linesPerPage As Integer = Convert.ToInt32(bodyR.Height / bodyFontHeight) - 1

        ' the header = 2 lines
        Dim currentLine As Integer = 2
        ' currentY is what we'll use for the Top (Y) coordinate that
        ' DrawString needs.
        Dim currentY As Single = 0

        ' Print each line of the file.  This could be easily modified to
        ' deal with data from a DataTable
        Do While currentLine + 1 < linesPerPage And _Data.Peek() > -1

            text = _Data.ReadLine()

            data = text.Split(",")

            ' retrieve the current top position
            currentY = getCurrentY(currentLine, bodyR.Top, bodyFontHeight)
            currentLine += 1

            ' DrawString will take a Rectangle argument, but NOT a RectangleF arg,
            ' so we need to provide the X and Y arguments.
            e.Graphics.DrawString(data(0), _bodyFont, _defaultBrush, bodyR.Left, currentY)
            currentY = getCurrentY(currentLine, bodyR.Top, bodyFontHeight)
            currentLine += 1
            e.Graphics.DrawString(data(1), _bodyFont, _defaultBrush, bodyR.Left + 25, currentY)
            currentY = getCurrentY(currentLine, bodyR.Top, bodyFontHeight)
            currentLine += 1
            e.Graphics.DrawString("Phone: " + data(2), _bodyFont, _defaultBrush, bodyR.Left + 25, currentY)
            currentY = getCurrentY(currentLine, bodyR.Top, bodyFontHeight)
            currentLine += 1
            e.Graphics.DrawString("Fax: " + data(3), _bodyFont, _defaultBrush, bodyR.Left + 25, currentY)
            'currentY = getCurrentY(currentLine, topMargin, bodyFontHeight)
            'currentLine += 1
            'ev.Graphics.DrawLine(_defaultPen, leftMargin, currentY, pageWidth, currentY)
            currentLine += 1
        Loop

        ' page number
        centerText(e.Graphics, "Page " + _currentPage.ToString(), _bodyFont, _defaultBrush, ReportFooterR)

        'Do we have more pages to print?
        If _Data.Peek() <> -1 Then
            e.HasMorePages = True
        Else
            e.HasMorePages = False
        End If
    End Sub
    Private Function getCurrentY(ByVal currentLine As Integer, ByVal topMargin As Single, ByVal fontHeight As Single) As Single
        Return topMargin + (currentLine * fontHeight)
    End Function

    Private Sub _PDoc_BeginPrint(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintEventArgs) Handles _PDoc.BeginPrint
        ' make sure we reset this before printing
        _currentPage = 0
        ' the file should be in the same folder as the EXE
        _Data = New StreamReader("contacts.csv")
        _headerFont = New Font("Arial", 24, FontStyle.Underline, GraphicsUnit.World)
        _bodyFont = New Font("Arial", 12, GraphicsUnit.World)
    End Sub

    Private Sub _PDoc_EndPrint(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintEventArgs) Handles _PDoc.EndPrint
        _Data.Close()
        _headerFont.Dispose()
        _bodyFont.Dispose()
    End Sub

    Private Sub _PDoc_QueryPageSettings(ByVal sender As Object, ByVal e As System.Drawing.Printing.QueryPageSettingsEventArgs) Handles _PDoc.QueryPageSettings

    End Sub

    Private Sub PrintButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles PrintButton.Click
        Dim pd As New PrintDialog()
        pd.Document = _PDoc
        Dim Rc As DialogResult = pd.ShowDialog()
        If Rc = DialogResult.OK Then _PDoc.Print()

    End Sub
    ' used for debugging
    Private Sub dumpRectF(ByVal rect As RectangleF)
        System.Diagnostics.Debug.WriteLine(rect.ToString())
    End Sub
    ' my wrapper for DrawRectangle
    Private Sub drawRect(ByVal g As Graphics, ByVal p As Pen, ByVal left As Single, ByVal top As Single, ByVal width As Single, ByVal height As Single)
        g.DrawRectangle(_defaultPen, CInt(left), CInt(top), CInt(width), CInt(height))
    End Sub
    ' the StringFormat class has a lot of cool features
    Private Sub centerText(ByVal g As Graphics, ByVal t As String, ByVal f As Font, ByVal b As Brush, ByVal rect As RectangleF)
        Dim sf As New StringFormat()
        ' center horizontally
        sf.Alignment = StringAlignment.Center
        ' center vertically
        sf.LineAlignment = StringAlignment.Center
        g.DrawString(t, f, b, rect, sf)
    End Sub
End Class
